@@ -5,9 +5,25 @@ use warnings;
 
 use Term::ShellUI;
 use Net::ManageSieve::Siesh;
-use File::Temp qw/tempfile/;
+use App::Siesh::Batch;
 
-our $VERSION = '0.11';
+our $VERSION = '0.20';
+
+sub read_config {
+    my ($class,$file) = @_;
+
+    if (!$file) {
+	require Config::Find;
+	$file =  Config::Find->find( name => 'siesh' );
+    }
+    return {} if ! $file;
+    require Config::Tiny;
+
+    my $config = Config::Tiny->read($file);
+    die $config->errstr() if ! $config;
+
+    return $config->{_};
+}
 
 sub run {
     my ( $class, %config ) = @_;
@@ -15,23 +31,32 @@ sub run {
     # Set defaults for Net::ManageSieve::Siesh construction
     $config{user} ||= $ENV{USER};
     $config{host} ||= 'imap';
+    $config{tls}  ||= 'auto';
 
-    my @params = ();
+    my @params;
 
-    foreach ( 'debug', 'port' ) {
+    foreach ( qw(debug port) ) {
         push @params, ucfirst($_), $config{$_}
           if defined $config{$_};
     }
 
-    push @params, 'tls', $config{'tls'}
-      if defined $config{'tls'};
+    ## Okay, this is counterintuative ... tls is the only options
+    ## for Net::ManageSieve contruction not ucfirsted.
+    ## TODO: Send in a patch
+    push @params, tls => $config{tls};
 
     my $sieve = Net::ManageSieve::Siesh->new( $config{host}, @params )
       or die "Can't connect to $config{host}: $!\n";
 
     $sieve->auth( $config{user}, $config{password} ) or die "$@\n";
 
+    my %shellui_params;
+    if ($config{file}) {
+	$shellui_params{term} = App::Siesh::Batch->new($config{file});
+    }
+
     my $term = new Term::ShellUI(
+	%shellui_params,
         history_file => '~/.siesh_history',
         prompt       => 'siesh> ',
         commands     => {
@@ -44,17 +69,15 @@ sub run {
                 desc    => 'Upload a script onto the server.',
                 maxargs => 2,
                 minargs => 2,
-                proc =>
-                  sub { $sieve->putfile(@_) or die $sieve->error() . "\n" },
-                args => sub { shift->complete_files(@_); },
+                proc => sub { $sieve->putfile(@_) },
+                args => sub { complete_file_and_script(@_, $sieve); },
             },
             "get" => {
                 desc    => "Fetch a script from the server and store locally.",
                 maxargs => 2,
                 minargs => 2,
-                proc =>
-                  sub { $sieve->getfile(@_) or die $sieve->error() . "\n" },
-                args => sub { shift->complete_files(@_); },
+                proc    => sub { $sieve->getfile(@_) or die $sieve->error() . "\n" },
+                args    => sub { complete_script_and_file(@_, $sieve); },
             },
             "quit" => {
                 desc    => "Quit siesh.",
@@ -64,21 +87,24 @@ sub run {
             "list" => {
                 desc    => "List all scripts stored on the server.",
                 maxargs => 0,
-                proc    => sub { $sieve->print_script_listing },
+                proc    => sub {
+    			my $active  = $sieve->get_active();
+    			my @scripts = $sieve->listscripts(1);
+    			print $active . " *\n" if $active;
+    			print join("\n",sort @scripts) . "\n" if @scripts;
+		},
             },
             "activate" => {
                 desc    => "Mark a script as active.",
                 maxargs => 1,
-                proc =>
-                  sub { $sieve->setactive(shift) or die $sieve->error() . "\n" }
-                ,
+                proc    => sub { $sieve->setactive(shift) },
                 args => sub { complete_scripts( @_, $sieve ) },
             },
             "edit" => {
                 desc    => 'Edit script using $EDITOR.',
                 maxargs => 1,
                 proc    => sub { $sieve->edit_script(shift) },
-                args => sub { complete_scripts( @_, $sieve ) },
+                args    => sub { complete_scripts( @_, $sieve ) },
             },
             "view" => {
                 desc    => 'Examine the contents of a script using $PAGER.',
@@ -89,13 +115,20 @@ sub run {
             "delete" => {
                 desc    => "Delete a script from the server.",
                 minargs => 1,
-                proc    => sub { $sieve->delete(shift) },
+                proc    => sub { 
+			if ( $_[0] eq '*' ) {
+				$sieve->deactivate();
+				$sieve->delete($sieve->listscripts);
+			} else {
+				$sieve->delete(@_);
+			}
+		},
                 args    => sub { complete_scripts( @_, $sieve ) },
             },
             "cat" => {
                 desc    => "Show the contents of a script on stdout.",
-                minargs => 1,
-                proc    => sub { $sieve->cat(shift) },
+                maxargs => 1,
+                proc    => sub { print $sieve->getscript(shift) },
                 args    => sub { complete_scripts( @_, $sieve ) },
             },
             "copy" => {
@@ -103,7 +136,7 @@ sub run {
                 maxargs => 2,
                 minargs => 2,
                 proc =>
-                  sub { $sieve->copyscript(@_) or die $sieve->error() . "\n" },
+                  sub { $sieve->copyscript(@_) },
                 args => sub { complete_scripts( @_, $sieve ) },
             },
             "move" => {
@@ -111,15 +144,14 @@ sub run {
                 maxargs => 2,
                 minargs => 2,
                 proc =>
-                  sub { $sieve->movescript(@_) or die $sieve->error() . "\n" },
+                  sub { $sieve->movescript(@_) },
                 args => sub { complete_scripts( @_, $sieve ) },
             },
             "deactivate" => {
                 desc    => 'Mark the currently activated script as inactive.',
                 maxargs => 0,
-                proc =>
-                  sub { $sieve->deactivate() or die $sieve->error() . "\n" },
-                args => sub { complete_scripts( @_, $sieve ) },
+                proc    => sub { $sieve->deactivate() },
+                args    => sub { complete_scripts( @_, $sieve ) },
             },
             "q"      => { alias => 'quit',       exclude_from_completion => 1 },
             "logout" => { alias => 'quit',       exclude_from_completion => 1 },
@@ -128,9 +160,7 @@ sub run {
             "dir"    => { alias => "list",       exclude_from_completion => 1 },
             "rm"     => { alias => "delete",     exclude_from_completion => 1 },
             "vi"     => { alias => "edit",       exclude_from_completion => 1 },
-            "less"   => { alias => "less",       exclude_from_completion => 1 },
             "more"   => { alias => "less",       exclude_from_completion => 1 },
-            "touch"  => { alias => "edit",       exclude_from_completion => 1 },
             "type"   => { alias => "cat",        exclude_from_completion => 1 },
             "cp"     => { alias => "copy",       exclude_from_completion => 1 },
             "mv"     => { alias => "move",       exclude_from_completion => 1 },
@@ -138,15 +168,38 @@ sub run {
             "unset"  => { alias => "deactivate", exclude_from_completion => 1 },
         },
     );
-
+    
+    #$term->{debug_complete}=5;
     $term->{term}->ornaments(0);
     $term->run();
 }
 
+sub complete_script_and_file {
+    my ( $term, $cmp, $sieve ) = @_;
+    if ($cmp->{argno} == 0 ) {
+	my $scripts = complete_scripts($term,$cmp,$sieve) ;
+	if ( @{ $scripts } ) {
+		return $scripts;
+	} else {
+		return "No scripts to complete found.\n";
+	}
+    } elsif ($cmp->{argno} == 1 ) {
+	return $term->complete_files($cmp) 
+    }
+}
+
+sub complete_file_and_script {
+    my ( $term, $cmp, $sieve ) = @_;
+    if ($cmp->{argno} == 0 ) {
+	return $term->complete_files($cmp) 
+    } elsif ($cmp->{argno} == 1 ) {
+		return "No scripts to complete found.\n";
+    }
+}
+
 sub complete_scripts {
-    my ( $self, $cmp, $sieve ) = @_;
-    return [ grep { index( $_, $cmp->{str} ) == 0 }
-          @{ $sieve->listscripts() } ];
+    my ( $term, $cmp, $sieve ) = @_;
+    return [ grep { index( $_, $cmp->{str} ) == 0 } $sieve->listscripts()  ];
 }
 
 1;
@@ -170,8 +223,9 @@ App::Siesh - interactive sieve shell
 
 =head1 DESCRIPTION
 
-App::Siesh provides a shell-like interface for manipulating sieve scripts
-using the ManageSieve protocol.
+App::Siesh provides a shell-like interface for manipulating sieve
+scripts using the ManageSieve protocol. If you search a command
+line utility, take a look at L<siesh>.
 
 =head1 OPTIONS
 
@@ -196,8 +250,13 @@ Specifies the remote port to connect to. Defaults to C<2000>.
 
 =item tls
 
-Specifies whether TLS is required ("require") or optional
-("auto"). Defaults to unset, disabling TLS.
+Specifies whether TLS is required ("require"), optional
+("auto") or disables ("off"). Defaults to I<auto>.
+
+=item file
+
+If an IO::Handle object is provdided, App::Siesh won't read commands
+from the command line prompt, but from that filehandle.
 
 =item password
 
@@ -214,14 +273,17 @@ Specifies the password to login.
 Prints a list of all scripts on the server. The currently active script,
 if any, is marked by a I<*> (astersik).
 
-Synonyms: B<ls>
+Synonyms: B<ls>, B<dir>
 
 =item B<delete> I<script-name> I<...>
 
-Deletes a script. It's not possible to delete the currently active
+Deletes all listed scripts. It's not possible to delete the currently active
 script, so please use deactivate first. There's no way to undelete a
 deleted script.
 
+If you specify I<*> (asterisk) as first argument to delete, the active
+script is deactivated and B<all> scripts are deleted.
+
 Synonyms: B<rm>
 
 =item B<edit> I<script-name>
@@ -235,6 +297,23 @@ this command.
 If your script is syntactical incorrect, you will be prompted to
 re-edit the file or throw away your changes.
 
+Synonyms: B<vi>
+
+=item B<copy> I<script-name> I<script-name>
+
+Copies the contents of the source script-name to a target  script.
+The contents of the target script are overridden.
+
+Synonyms: B<cp>
+
+=item B<move> I<script-name> I<script-name>
+
+Moves script to a destination script. The destination script is
+overridden.
+
+Synonyms: B<mv>
+
+
 =item B<activate> I<script-name>
 
 Activates the listed script. User may have multiple Sieve scripts on
@@ -242,24 +321,41 @@ the server, yet only one script may be used for filtering of incoming
 messages. This is called the active script. Users may have zero or one
 active scripts
 
+Synonyms: B<set>
+
 =item B<deactivate>
 
 Deactivate all scripts. Deactivation of all your scripts results in no
 filtering at all.
 
-=item B<cat> I<script-name> I<...>
+Synonyms: B<unset>
 
-Print scripts on the standard output.
+=item B<cat> I<script-name> 
+
+Print script on the standard output.
+
+Synonyms: B<type>
+
+=item B<view> I<script-name>
+
+Calls $ENV{PAGER} or "less" on script. In case of any error, we fall
+back to use cat.
+
+Synonyms: B<more>
 
 =item B<quit>
 
 Terminates the sessiion with the remote SIEVE server. An end of file
 will also terminate the session and exit.
 
+Synonyms: B<q>, B<logout>
+
 =item B<help>
 
 Print a short description of all commands.
 
+Synonyms: B<h>
+
 =item B<put> I<file-name> I<script-name>
 
 Store a local file as script on the remote machine.
